/* PCS CAD Commander Update Procedure */
'@echo off'
call RxFuncAdd 'SysLoadFuncs','RexxUtil','SysLoadFuncs'
call SysLoadFuncs
call SysCls

/* Note on PATCHIT:
   PATCHIT has the following return codes:
    0  = Successful patch completed or file already up to date
    > 0 = OS/2 File Open error
    -1 = Not PATCHIT (.PIT) file specified
    -2 = Patch file load failed (not found)
    -3 = Unable to access file specified in .PIT file
    -4 = Invalid version of file found - unable to patch it
    -5 = Error loading file to patch (file open or error allocating RAM to hold file)
    -6 = Unknown/unspecified internal PATCHIT error (patch failed)
*/

/* If changing README.14 length, modify display of README filename in final msg */
readmefile = 'README.14'
emailsubject = 'CADCMDR 14'
updver = '1.4'
orgver = '1.2/1.3x'
ActiveExt = ''
szResult = ''
fPatchKeyboard = 1
fPatchDaemon = 1
fPatchShell = 1
fUpdateWPSOBject = 0
fPatchDocumentation = 1
fRecreateDesktopObjects = 0

/* Primary patch files */
CadcPatchFiles = 'CC12-14.PIT CC13-14.PIT'
CadcExtensions = '152 153' /* Original extensions */
PkmdPatchFiles = 'PD12-14.PIT PD13-14.PIT PD131-14.PIT'
PkmdExtensions = '12 13 131'
KbddPatchFiles = 'KD13-14.PIT'
KbddExtensions = '12'
DocPatchFiles = 'INF12-13.PIT'
DocExtensions = '12'


/* Setup header for error logging messages with today's date */
today = Date( )': '

/* Verify this was started in the CAD Commander directory */
szCurrentDirectory = directory()
PARSE UPPER VAR szCurrentDirectory szCurrentDirectory
if SUBSTR( szCurrentDirectory, 2 ) <> ':\CADCMDR' then
DO
 /* 'Echo 'today'CADCMDR Update failed due to wrong starting directory >>\CADCMDR\CADCUPDT.LOG' */
 BEEP( 1000, 250 )
 SAY ' '
 SAY ' '
 SAY ' '
 SAY '-------------------------------------------------------------------------------'
 SAY 'You must start the CCPATCH procedure from within the CADCMDR directory'
 SAY '-------------------------------------------------------------------------------'
 exit
END


/* Setup log file and unpatch file */
logfile = szCurrentDirectory'\CADCUPDT.LOG'
unpfile = szCurrentDirectory'\CCUNPAT.CMD'


SAY 'PCS Ctrl-Alt-Del Commander 'updver' Patch Procedure'
SAY 'Updates version 'orgver' to 'updver
SAY '(c) 1995-1996 Perez Computing Services.  All Rights Reserved.'

/* Locate the \OS2\BOOT directory */
cBootDrive = ' '
cBootDrive=Left( Value( 'RUNWORKPLACE',,'OS2ENVIRONMENT' ), 1 )


/* This will erase old CADCUPDT.LOG file and start a new one */
szTime = Time( 'C' )
szDate = Date()
'Echo 'szDate', 'szTime': Beginning update of Ctrl-Alt-Del Commander 'orgver' to 'updver' >'logfile



/* SIGNAL FINAL_MSG */

SAY 'Beginning patching process, please wait...'

/* Make sure \CADCMDR\UPDATE directory exists */
call SysMkDir szCurrentDirectory'\update'

SAY 'Modifying files to be patched attributes'
'ATTRIB -s -r -h *.* >NUL'
'ATTRIB -s -r -h UPDATE\*.* >NUL'
'ATTRIB -s -r -h PATCH\*.* >NUL'

/* Move current PKM_DMN.EXE to UPDATE directory for patching */
 IF fPatchDaemon = 1 THEN CALL COPYFILE 'PKM_DMN.EXE'

/* Move current CADCMDR.EXE to UPDATE directory for patching */
 IF fPatchShell = 1 THEN CALL COPYFILE 'CADCMDR.EXE'


/* Create new CCUNPAT.CMD file */
CALL CreateNewUnpatchFile


/* Patch online docs  */
IF fPatchDocumentation = 1 THEN CALL PatchDocumentation



/********************************************************************/
/* Patch KBDBASE.SYS                                                */
/********************************************************************/
IF fPatchKeyboard = 1 THEN CALL PatchDriver

/********************************************************************/
/* Switch to UPDATE directory to do patching of locked files */
/********************************************************************/
IF (fPatchDaemon = 1) | (fPatchShell = 1) THEN
DO
 call directory( szCurrentDirectory'\UPDATE' )

 /********************************************************************/
 /* Patch CADCMDR.EXE in UPDATE directory                            */
 /********************************************************************/
 IF fPatchShell = 1 THEN CALL PatchShell

 /********************************************************************/
 /* Patch PKM_DMN.EXE in UPDATE directory                            */
 /********************************************************************/
 IF fPatchDaemon = 1 THEN CALL PatchDaemon
END


/********************************************************************/
/* Discard patch files, we're done using them                       */
/********************************************************************/
SAY 'Cleaning up patch files'
call directory( szCurrentDirectory )
'DEL PATCH\*.PIT > NUL'


/********************************************************************/
/* Create Ctrl-Alt-Del Commander Control Panel WPS Object           */
/********************************************************************/
 IF fUpdateWPSOBject = 1 THEN
 DO
    SAY 'Creating WPS Ctrl-Alt-Del Commander Control Panel'
    CALL InstallControlPanel
 END


/* Display the termination message */
FINAL_MSG:

SAY ' '
SAY ' '
SAY ''
SAY '              Ctrl-Alt-Del Commander Patching Procedure Complete             '
SAY '                                                                             '
SAY ' Patching procedure complete.  Please shutdown and restart your system       '
SAY ' to allow changes to take affect.  Please be sure to read the "Problem       '
szMsg = 'Determination" section of the 'readmefile' if you have difficulties with'
DO WHILE LENGTH( szMsg ) < 76
    szMsg = szMsg' '
END
SAY ' 'szMsg''

SAY ' this version.  If, after trying the suggestions provided, you still         '
SAY ' have problems please contact PCS Technical Support at "pcs@ncia.com".       '
SAY ' Please provide a copy of your CONFIG.SYS and a list of any WPS              '
SAY ' add-ons (enhancement products) you may be using.  We also look forward      '
SAY ' to getting suggestions from our customers on how to improve our products.   '
SAY '                                                                             '

szMsg = 'Thank you for using the Ctrl-Alt-Del Commander 'updver'.'
DO WHILE LENGTH( szMsg ) < 76
    szMsg = szMsg' '
END
SAY ' 'szMsg''


szMsg = 'Please provide feedback to "pcs@ncia.com" subject "'emailsubject'".'
DO WHILE LENGTH( szMsg ) < 76
    szMsg = szMsg' '
END
SAY ' 'szMsg''
SAY '                                                                             '
SAY ' A log file (CADCUPDT.LOG) has been created showing updated files.           '
SAY '                                                                             '
SAY ' You must have the CALL='cBootDrive':\OS2\CMD.EXE /C 'szCurrentDirectory'\CADUPDT.CMD in your     '
SAY ' CONFIG.SYS for the updated changes to be installed following a reboot.      '
SAY '                                                                             '
SAY ''
EXIT


/* Copy file the specified file into the UPDATE directory */
COPYFILE:
szCurFile = ARG( 1 )
SAY 'Copying 'szCurFile' to \CADCMDR\UPDATE directory for service.'
'COPY 'szCurFile' UPDATE\'szCurFile'/B >NUL'
TmpRc = rc
IF RC <> 0 THEN
DO
 'Echo 'today'Failure copying 'szCurFile' to update directory so it can be patched: 'TmpRc'>>'logfile
 BEEP( 1000, 250 )
 SAY ' '
 SAY ' '
 SAY ' '
 SAY '-------------------------------------------------------------------------------'
 SAY 'Failure copying 'szCurFile' to update directory so it can be patched (rc: 'TmpRc').'
 SAY 'You may not have write access to this drive or the drive may be full.'
 SAY '-------------------------------------------------------------------------------'
 EXIT
END
RETURN



/* Create a new unpatch file (all other references simply append to it) */
CreateNewUnpatchFile:
'Echo @echo off >'unpfile
RETURN


/* Perform the keyboard driver patching */
PatchKeyboardDriver:
patchfile = ARG( 1 )
ActiveExt = ARG( 2 )
'Echo 'today'Patching KBDBASE.SYS to 'updver' level >>'logfile
SAY 'Patching PCS KBDBASE.SYS driver to 'updver' level '
'PATCHIT PATCH\'patchfile
TmpRc = RC

/* If RC = -4, then the first patch file found invalid version to patch, so lets
    try the second (alternate) patch file */
IF TmpRc = -4 THEN RETURN

IF TmpRc <> 0 THEN
DO
  BEEP( 1000, 250 )
  echo '"Dooooeeeeh!"  Patching of KBDBASE.SYs failed error: 'TmpRc'>>'logfile
  SAY ' '
  SAY ' '
  SAY ' '
  SAY '-------------------------------------------------------------------------------'
  SAY 'Patching of KBDBASE.SYS failed, error: 'TmpRc'.  This update will not work correctly.'
  CALL GetPatchError TmpRc
  SAY 'Error "'szResult'".'
  SAY 'Aborting update procedure.'
  SAY ' '
  SAY 'Be sure that the CADCMDR 'orgver' KBDBASE.SYS is located in 'szCurrentDirectory' and'
  SAY 'retry the update procedure.  You can copy it from 'cBootDrive':\OS2\BOOT if '
  SAY 'Ctrl-Alt-Del Commander is currently running on your system.'
  SAY '-------------------------------------------------------------------------------'
  EXIT
END
RETURN


/* Archive the previous level of the KBDBASE.SYS driver */
ArchivePatchedKeyboardDriver:
  szPath = 'KBDBASE.PBK'
  call SysFileTree szPath, 'fileinfo', 'F'
  IF fileinfo.0 <> 0 THEN
  DO
     'COPY KBDBASE.PBK KBDBASE.'ActiveExt' /B >NUL'
     'DEL KBDBASE.PBK >NUL'

     echo 'COPY KBDBASE.'ActiveExt' KBDBASE.SYS /B >>'unpfile
     echo 'COPY KBDBASE.SYS 'cBootDrive':\OS2\BOOT\KBDBASE.SYS /B >>'unpfile
     SAY 'Installing KBDBASE.SYS driver in the 'cBootDrive':\OS2\BOOT directory'
     SAY '(Your original remains safely in the 'cBootDrive':\OS2\BOOT\KBDBASE.00x'
     SAY ' file).'
     SAY 'The KBDBASE.SYS for version 'orgver' has been renamed to KBDBASE.'ActiveExt
  END
RETURN

/* Actually copy the newly patched keyboard driver into the OS2\BOOT directory */
InstallNewKeyboardDriver:
'COPY KBDBASE.SYS 'cBootDrive':\OS2\BOOT /B >NUL'
IF RC <> 0 THEN
DO
 'echo 'today'"Dooooeeeeh!"  Copying of KBDBASE.SYS to 'cBootDrive':\OS2\BOOT failed due to error: 'rc'.>>'logfile
 BEEP( 1000, 250 )
 SAY ' '
 SAY ' '
 SAY ' '
 SAY '-------------------------------------------------------------------------------'
 SAY 'Copying of KBDBASE.SYS to 'cBootDrive':\OS2\BOOT failed.  This update will '
 SAY 'not work correctly.  Aborting update procedure.'
 SAY 'You may not have write access to the drive or the drive may be full.'
 SAY 'Run CCUNPAT.CMD to return to Ctrl-Alt-Del Commander 'orgver'.'
 SAY '-------------------------------------------------------------------------------'
 EXIT
END
RETURN


/* Patch the file whose name, description, patch file, and extension are passed in */
PatchFile:
 PARSE ARG szCurFile,szDescription,szPatchFile,ActiveExt
/*  SAY 'szCurFile: 'szCurFile
 SAY 'Desc: 'szDescription
 SAY 'PatchFile: 'szPatchFile
 SAY 'Ext: 'ActiveExt */
 SAY 'Patching of file: 'szDescription' ('szCurFile'.EXE)'
 '..\PATCHIT ..\PATCH\'szPatchFile' /Q'
 TmpRc = rc
 IF TmpRc = -4 THEN RETURN

 /* Handle an error trying to patch file */
 IF TmpRc <> 0 THEN
 DO
  'echo 'today'"Doehhhh!"  'szCurFile'.EXE patch to version 'updver' failed due to error: 'TmpRc'.>>'logfile
  BEEP( 1000, 250 )
  SAY ' '
  SAY ''
  SAY ' '
  SAY '-------------------------------------------------------------------------------'
  SAY 'Patching process failed due to error: 'TmpRc'.'
  SAY 'Run CCUNPAT.CMD to return to Ctrl-Alt-Del Commander 'orgver'.'
  SAY '-------------------------------------------------------------------------------'
  EXIT
 END
 ELSE
 DO
  /* Hurray - We've patched it (or it didn't need it) */
  call SysFileTree szCurFile'.PBK', 'fileinfo', 'F'
  IF fileinfo.0 <> 0 THEN
  DO
      'echo DEL \CADCMDR\UPDATE\'szCurFile'.EXE >'unpfile
      'echo 'today''szCurFile'.EXE patched to version 'updver'.>>'logfile
      'copy 'szCurFile'.PBK ..\'szCurFile'.'ActiveExt' /B >NUL'
      'DEL 'szCurFile'.PBK > NUL'
      SAY 'The 'orgver' 'szCurFile'.EXE has been renamed to 'szCurFile'.'ActiveExt
  END
  ELSE 'echo 'today''szCurFile'.EXE already at 'updver'.>>'logfile
  RETURN
END

/* Create the CADCMDR Control Panel */
InstallControlPanel:
    /* Delete the old icon */
    rc = SysDestroyObject( 'CADC_WPS' )

    /* Deregister the class from WPS */
    rc = SysDeregisterObjectClass( 'CADC_WPS' );     /* Deregister CADC_WPS class */

    /* Now we can copy files down to this directory */
    'COPY 'szCurrentDirectory'\UPDATE\CADCRES.DLL /B >NUL'
    'COPY 'szCurrentDirectory'\UPDATE\CADC_WPS.DLL /B >NUL'
    'COPY 'szCurrentDirectory'\UPDATE\CADC_WPS.HLP /B >NUL'

     rc = SysRegisterObjectClass( 'CADC_WPS', szCurrentDirectory'CADC_WPS.DLL' );

     /* Now create object on Desktop */
     rc = SysCreateObject( 'CADC_WPS', 'Ctrl-Alt-Del Commander^Control Panel', '<WP_DESKTOP>', ,
                        'OBJECTID=<CADC_WPS>;',,
                        'replace' );

RETURN


/* Convert PATCHIT error to a string */
GetPatchError:
 IF ARG( 1 ) = 0 THEN szResult = 'OK'
 IF ARG( 1 ) > 1 THEN szResult = 'OS/2 File Error'
 IF ARG( 1 ) = -1 THEN szResult = 'PATCHIT Patch File not specified'
 IF ARG( 1 ) = -2 THEN szResult = 'PATCHIT Patch File not found'
 IF ARG( 1 ) = -3 THEN szResult = 'PATCHIT Patch File could not be accessed.'
 IF ARG( 1 ) = -4 THEN szResult = 'Target patch file cannot be patched due to incorrect version'
 IF ARG( 1 ) = -5 THEN szResult = 'Patch file load error or not enough RAM'
 IF ARG( 1 ) = -6 THEN szResult = 'Unknown PATCHIT Internal error'
 RETURN


/* Create desktop objects */
CreateDesktopObjects:
NOP
RETURN


/* Begin process of patch of Keyboard Driver */.
PatchDriver:
lFile = 1;
lIndex = WORDINDEX( KbddPatchFiles, lFile )
DO WHILE lIndex <> 0
   szKbdPatchFile = SUBWORD( KbddPatchFiles, lFile, 1 )
   szExt = SUBWORD( KbddExtensions, lFile, 1 )
   CALL PatchKeyboardDriver szKbdPatchFile, szExt

   /* Exit immediately if no target file to look at */
   IF TmpRc = 0 THEN
   DO
     /********************************************************************/
     /* Handle result of patching KBDBASE.SYS driver                     */
     /********************************************************************/
     IF TmpRc = 0 THEN CALL ArchivePatchedKeyboardDriver
     ELSE
     DO
        SAY 'KBDBASE.SYS already up to date.'
        RETURN
     END

     /********************************************************************/
     /* Make sure we can copy to boot drive's OS2\BOOT directory */
     /********************************************************************/
     'ATTRIB -s -r -h 'cBootDrive':\OS2\BOOT\KBDBASE.SYS >NUL'

     /********************************************************************/
     /* Copy new KBDBASE.SYS */
     /********************************************************************/
     CALL InstallNewKeyboardDriver
     'echo 'today'New KBDBASE.SYS successfully placed in 'cBootDrive':\OS2\BOOT directory >>'logfile
     SAY 'New KBDBASE.SYS successfully placed in 'cBootDrive':\OS2\BOOT directory'

     RETURN
   END
   lFile = lFile + 1
   lIndex = WORDINDEX( KbddPatchFiles, lFile )
END
RETURN


/* Begin process of patch of CADCMDR file */
PatchShell:
lFile = 1;
lIndex = WORDINDEX( CadcPatchFiles, lFile )
DO WHILE lIndex <> 0
   Call PatchFile 'CADCMDR', 'Replacement PM Shell', SUBWORD( CadcPatchFiles, lFile, 1 ), SUBWORD( CadcExtensions, lFile, 1 )

   /* Exit immediately if no target file to look at */
   IF TmpRc = 0 THEN RETURN

   lFile = lFile + 1
   lIndex = WORDINDEX( CadcPatchFiles, lFile )
END
RETURN


/* Begin process of patch of PKM_DMN.EXE */
PatchDaemon:
SAY 'Patching daemon'
lFile = 1;
lIndex = WORDINDEX( PkmdPatchFiles, lFile )
DO WHILE lIndex <> 0
   szDaemonPatchFile = SUBWORD( PkmdPatchFiles, lFile, 1 )
   SAY 'Trying to use patch file 'szDaemonPatchFile
   Call PatchFile 'PKM_DMN', 'Replacement Keyboard Daemon', szDaemonPatchFile, SUBWORD( PkmdExtensions, lFile, 1 )

   /* Exit immediately if no target file to look at */
   IF TmpRc = 0 THEN RETURN

   lFile = lFile + 1
   lIndex = WORDINDEX( PkmdPatchFiles, lFile )
END
RETURN

/* Patch the documentation file */
PatchDocumentation:
lFile = 1;
lIndex = WORDINDEX( DocPatchFiles, lFile )
DO WHILE lIndex <> 0
   szDocPatchFile = SUBWORD( DocPatchFiles, lFile, 1 )
   szActiveExt = SUBWORD( DocExtensions, lFile )

   SAY 'Patching of file: Online Documentation (CADCMDR.INF)'
   'PATCHIT PATCH\'szDocPatchFile' /Q'
   TmpRc = rc

   /* Handle an error trying to patch file */
   IF (TmpRc <> 0) & (TmpRc <> -4) THEN
   DO
     'echo 'today'"Doehhhh!"  CADCMDR.INF patch to version 'updver' failed due to error: 'TmpRc'.>>'logfile
     BEEP( 1000, 250 )
     SAY ' '
     SAY ''
     SAY ' '
     SAY '-------------------------------------------------------------------------------'
     SAY 'Patching process of online documentation failed due to error: 'TmpRc'.'
     SAY 'Run CCUNPAT.CMD to return to Ctrl-Alt-Del Commander 'orgver'.'
     SAY '-------------------------------------------------------------------------------'
     EXIT
   END
   ELSE
   IF TmpRc = 0 THEN
   DO
     /* Hurray - We've patched it (or it didn't need it) */
     call SysFileTree 'CADCMDR.PBK', 'fileinfo', 'F'
     IF fileinfo.0 <> 0 THEN
     DO
         'copy CADCMDR.PBK CADCMDR.'szActiveExt' /B >NUL'
         'DEL CADCMDR.PBK > NUL'
         SAY 'The 'orgver' CADCMDR.PBK has been renamed to CADCMDR.'szActiveExt
         RETURN
     END
     ELSE
     DO
        'echo 'today'CADCMDR.INF documentation already at 'updver'.>>'logfile
        RETURN
     END
   END

   lFile = lFile + 1
   lIndex = WORDINDEX( DocPatchFiles, lFile )
END

